# -*- coding: utf8 -*-
import unittest
import shutil
import os
import time

from Page import *
import Repository
import Category
import Permission

class PageTestCase(unittest.TestCase):
	def setUp(self):
		try: Repository.removeRepository()
		except: pass
		Repository.createRepository()

		self.pageName = 'First Page'
		self.rawText = 'Hello World'
		self.category = Category.Category()
		self.filePath = os.path.join(self.category.dirPath, utils.encode(self.pageName))

	def tearDown(self):
		Repository.removeRepository()

	def test_createPage(self):
		if os.path.exists(self.filePath):
			os.remove(self.filePath)
		page = createPage(self.category, self.pageName, self.rawText)
		self.assert_(isinstance(page, Page))
		self.assert_(os.path.isfile(self.filePath))
	def test_CreatedPagePermission(self):
		page = createPage(self.category, self.pageName, self.rawText, perms='111000')
		self.assertEquals(page.perms.toString(), '111000')
		self.assertEquals(page.perms, '111000')

		os.remove(self.filePath)

		page = createPage(self.category, self.pageName, self.rawText, perms=Permission.Permission('000111'))
		self.assertEquals(page.perms.toString(), Permission.Permission('000111'))
		self.assertEquals(page.perms, Permission.Permission('000111'))

class PageClassTestCase(unittest.TestCase):
	def setUp(self):
		try: Repository.removeRepository()
		except: pass
		Repository.createRepository()

		self.pageName = 'Hello World'
		self.rawText = "I'm your father"
		self.category = Category.Category()
		filePath = os.path.join(self.category.dirPath, utils.encode(self.pageName))
		if os.path.exists(filePath):
			os.remove(filePath)
		self.assert_(createPage(self.category, self.pageName, self.rawText))

	def tearDown(self):
		Repository.removeRepository()

	def test_Init(self):
		page = Page(self.category, self.pageName)
		self.assertEquals(page.pageName, self.pageName)
		self.assertEquals(page.category, self.category)
		self.assertEquals(page.tags, '')
		self.assertEquals(page.encPageName, utils.encode(self.pageName))
		self.assert_(os.path.isfile(page.filePath))
		self.assertEquals(page.rawText, self.rawText)
		
		pageName = 'King Kong'
		tags = 'Movie, Animal'
		rawText = 'so pretty!'
		createPage(self.category, pageName, rawText, tags)
		page = Page(self.category, pageName)
		self.assertEquals(page.tags, tags)

		
	def test_Load(self):
		page = Page(self.category, self.pageName)
		self.assert_(os.path.isfile(page.filePath))
		page.load()
		self.assertEquals(page.rawText, self.rawText)
		#self.assertEquals(page.category.categoryPath, self.category.categoryPath)

	def test_Save(self):
		pageName = 'Page 1'
		rawText = 'raw text'
		editedText = 'edited'
		createPage(self.category, pageName, rawText)
		page = Page(self.category, pageName)
		page.rawText = editedText
		page._save()
		category = Category.Category()
		newPage = category.getPage(pageName)
		self.assertEquals(newPage.rawText, editedText)

	def test_ModifiedTime(self):
		pageName = 'Page 1'
		rawText = 'raw text'
		editedText = 'edited'
		page = createPage(self.category, pageName, rawText)
		mtime1 = str(os.path.getmtime(page.filePath))
		self.assertEquals(page.mtime, mtime1)
		time.sleep(1)
		page._save()
		mtime2 = str(os.path.getmtime(page.filePath))
		self.assertEquals(page.mtime, mtime2)

	def test_Edit(self):
		pageName = 'Page 1'
		rawText = 'raw text'
		editedText = 'edited'
		page = createPage(self.category, pageName, rawText)
		self.assertEquals(page.rawText, rawText)
		
		editedText = 'editedText'
		editedTags = 'editedTags'
		page.edit(editedText, editedTags)
		self.assertEquals(page.rawText, editedText)

	def test_Remove(self):
		page = Page(self.category, self.pageName)
		self.assert_(os.path.exists(page.filePath))
		self.assert_(page.remove())
		self.failIf(os.path.exists(page.filePath))
		self.assertRaises(NotExistPage, page.remove)

	def xtest_EditConflict(self): # FIXME webinterface에서 검사하면서 page에서 빠졌음.
		pageName = 'Page 1'
		rawText = 'raw text'
		editedText = 'edited'
		page = createPage(self.category, pageName, rawText)
		mtime = os.path.getmtime(page.filePath)
		self.assertEquals(page.rawText, rawText)
		
		editedText = 'editedText'
		editedTags = 'editedTags'
		page.edit(editedText, editedTags, writedTime=mtime+10)
		self.assertEquals(page.rawText, editedText)
		self.assertRaises(EditConflict, page.edit, editedText, editedTags, '', mtime-10)
	def test_Backup(self):
		pageName = 'Page 1'
		rawText = 'raw text'
		page = createPage(self.category, pageName, rawText)
		page.backup()
		backupRepository = Repository.getBackupRepository()
		cur, dirs, files = os.walk(backupRepository).next()
		mtime = str(int(os.path.getmtime(page.filePath)))
		self.assertEquals(files, ['Page_201.%s' % mtime])

		childCategory = self.category.createCategory('QueSeraSera')
		pageName = 'Page 2'
		rawText = 'raw text'
		page = createPage(childCategory, pageName, rawText)
		page.backup()
		backupRepository = os.path.join(backupRepository, utils.encode(childCategory.categoryName))
		cur, dirs, files = os.walk(backupRepository).next()
		mtime = str(int(os.path.getmtime(page.filePath)))
		self.assertEquals(files, ['Page_202.%s' % mtime])
	def test_GetGroup(self):
		pageName = 'Page 1'
		rawText = 'raw text'
		group = ['jangc', 'python']
		self.category.editGroup(group)
		page = createPage(self.category, pageName, rawText)
		self.assertEquals(page.getGroup(), group)

if __name__ == '__main__':
	unittest.main(argv=('','-v'))
