# -*- coding: utf8 -*-
import unittest
import shutil
import os

from Category import *
import Repository
import Page
import Tag

class CategoryTestCase(unittest.TestCase):
	def setUp(self):
		Repository.createRepository()

	def tearDown(self):
		Repository.removeRepository()

	def test_RootCategory(self):
		root = Category()
		self.assertEquals(root.categoryName, '')
		self.assertEquals(root.categoryPath, '')
		self.assertEquals(root.dirPath, Repository.getPageRepository())
		self.assert_(os.path.isdir(Repository.getPageRepository()))
		self.assertEquals(root.group, [])

	def test_LoadPermission(self):
		root = Category()
		file(os.path.join(root.dirPath, '__Attr__'), 'wb').write('111111')
		root._loadPerms()
		self.assertEquals(root.perms['member'].write, True)
		self.assertEquals(root.perms['member'].read, True)
		self.assertEquals(root.perms['group'].write, True)
		self.assertEquals(root.perms['group'].read, True)
		self.assertEquals(root.perms['anony'].write, True)
		self.assertEquals(root.perms['anony'].read, True)

		file(os.path.join(root.dirPath, '__Attr__'), 'wb').write('000000')
		root._loadPerms()
		self.assertEquals(root.perms['member'].write, False)
		self.assertEquals(root.perms['member'].read, False)
		self.assertEquals(root.perms['group'].write, False)
		self.assertEquals(root.perms['group'].read, False)
		self.assertEquals(root.perms['anony'].write, False)
		self.assertEquals(root.perms['anony'].read, False)
	def test_EditPermission(self):
		root = Category()

		root._editPerms('000000')
		self.assertEquals(root.perms['member'].write, False)
		self.assertEquals(root.perms['member'].read, False)
		self.assertEquals(root.perms['group'].write, False)
		self.assertEquals(root.perms['group'].read, False)
		self.assertEquals(root.perms['anony'].write, False)
		self.assertEquals(root.perms['anony'].read, False)

		root._editPerms('111111')
		self.assertEquals(root.perms['member'].write, True)
		self.assertEquals(root.perms['member'].read, True)
		self.assertEquals(root.perms['group'].write, True)
		self.assertEquals(root.perms['group'].read, True)
		self.assertEquals(root.perms['anony'].write, True)
		self.assertEquals(root.perms['anony'].read, True)



	def test_JoinCategoryPath(self):
		root = Category()
		self.assertEquals(root.joinCategoryPath('My eggs'), 'My eggs')


	def test_createCategory(self):
		categoryName = 'Show me the money'
		root = Category()
		category = root.createCategory(categoryName)
		self.assertEquals(category.categoryName, categoryName)
		self.assertEquals(category.group, [])
		self.assert_(os.path.isdir(category.dirPath))
		self.assert_(os.path.isfile(os.path.join(category.dirPath, '__Attr__')))
		self.assertEquals(category.categoryPath, categoryName)

		categoryName2 = 'One Won'
		subCategory = category.createCategory(categoryName2)
		self.assertEquals(subCategory.categoryName, categoryName2)
		self.assert_(os.path.isdir(subCategory.dirPath))
		self.assertEquals(subCategory.group, [])
		self.assertEquals(subCategory.categoryPath, categoryName+'/'+categoryName2)

	def test_CreateCategoryByCategoryPath_PermissionNone(self):
		category = Category()
		category = createCategoryByCategoryPath('Best/Wiki')
		self.assertEquals(category.perms, category.perms)
		self.assertEquals(category.parent.perms, category.perms)

	def test_CreateCategoryByCategoryPath_Permission(self):
		perms = Permission.Permission('111000')
		category = createCategoryByCategoryPath('Best/Wiki', perms)
		self.assertNotEquals(perms, Category().perms)
		self.assertEquals(category.perms, perms)
		self.assertEquals(category.parent.perms, perms)

	def test_CreateCategory_Permission(self):
		category = Category()
		perms = Permission.Permission('000000')
		subCategory = category.createCategory('permTest', perms)
		self.assertEquals(subCategory.perms, perms)

		subCategory = category.createCategory('permTest2', perms)
		self.assertEquals(subCategory.perms, perms)

	def test_Remove(self):
		category = Category()

		sun = category.createCategory('Sun')
		self.assertEquals(sun.categoryName, 'Sun')
		sun.remove()
		self.failIf(category.getCategory('Sun'))
		self.failIf(os.path.exists(sun.dirPath))

		sun = category.createCategory('Sun')
		self.assertEquals(sun.categoryName, 'Sun')
		page = Page.createPage(sun, 'Sun', 'text', 'My Books, Mine')
		sun.remove()
		self.failIf(category.getCategory('Sun'))
		self.failIf(os.path.exists(sun.dirPath))
		self.failIf(Tag.Tag('My Books').getPagePathList())
		self.failIf(Tag.Tag('Mine').getPagePathList())
		
		sun = category.createCategory('Sun')
		category.remove()
		self.failIf(category.getCategoryList())
		self.assert_(os.path.exists(category.dirPath))




	def test_GetPageList(self):
		root = Category()
		Page.createPage(root, 'Books', 'JustForFun\n서유기\nYou can negotiate anything')
		pageList = root.getPageList()
		self.assertEquals(len(pageList), 1)
		page = pageList[0]
		self.assertEquals(page.pageName, 'Books')

		pageName = 'Book %s' 
		for i in range(10):
			Page.createPage(root, pageName % i, 'JustForFun\n서유기\nYou can negotiate anything')

		pageList = root.getPageList()
		self.assertEquals(len(pageList), 11)

		for i in range(4):
			Page.createPage(root, 'Han %s' % i, 'text')
		myFilter = lambda page:page.pageName.startswith('Han')
		pageList = root.getPageList(myFilter)
		self.assertEquals(len(pageList), 4)

	def test_GetCategoryList(self):
		root = Category()
		root.createCategory('Foods')
		categoryList = root.getCategoryList()
		self.assertEquals(len(categoryList), 1)
		category = categoryList[0]
		self.assertEquals(category.categoryName, 'Foods')

		categoryName = 'Foods %s' 
		for i in range(10):
			category.createCategory(categoryName % i)
		categoryList = category.getCategoryList()
		self.assertEquals(len(categoryList), 10)

	def test_GetCategory(self):
		categoryName = 'Foods'
		root = Category()
		category = root.getCategory(categoryName)
		self.assertEquals(category, None)

		root.createCategory(categoryName)
		category = root.getCategory(categoryName)
		self.assert_(category)

		self.failIf(root.getCategory(''))
		self.failIf(root.getCategory('/'))
		self.failIf(root.getCategory('//'))

	def test_CreateCategoryByCategoryPath(self):
		categoryPath = 'Foods/Apple'
		createCategoryByCategoryPath(categoryPath)
		category = Category()
		for name in categoryPath.split('/'):
			category = category.getCategory(name)
			self.assert_(category)


	def test_GetCategoryByCategoryPath(self):
		categoryPath = ''
		category = getCategoryByCategoryPath(categoryPath)
		self.assertEquals(category.categoryPath, Category().categoryPath)
		self.assertEquals(category.parent, None)

		categoryPath = '/'
		category = getCategoryByCategoryPath(categoryPath)
		self.assertEquals(category.categoryPath, Category().categoryPath)
		self.assertEquals(category.parent, None)

		categoryPath = 'Foods/Apple'
		category = getCategoryByCategoryPath(categoryPath)
		self.assertEquals(category, None)

		category = Category()
		for name in categoryPath.split('/'):
			category = category.createCategory(name)
		del category
		category = getCategoryByCategoryPath(categoryPath)
		self.assert_(category)

	def test_GetCategoryByCategoryPath_Exist(self):
		createCategoryByCategoryPath('Cate1/Cate2/Cate3')
		category = getCategoryByCategoryPath('Cate1/NotExist', exist=True)
		self.assertEquals(category.categoryName, 'Cate1')

		category = getCategoryByCategoryPath('Cate1/Cate2/NotExist', exist=True)
		self.assertEquals(category.categoryName, 'Cate2')
		category = getCategoryByCategoryPath('Cate1/Cate2/Cate3/NotExist', exist=True)
		self.assertEquals(category.categoryName, 'Cate3')

		category = getCategoryByCategoryPath('Cate1/Cate2/Cate3/NotExist/NotExist2', exist=True)
		self.assertEquals(category.categoryName, 'Cate3')

		category = getCategoryByCategoryPath('Cate1/Cate2/Cate3', exist=True)
		self.assertEquals(category.categoryName, 'Cate3')

		category = getCategoryByCategoryPath('Cate1/Cate2/', exist=True)
		self.assertEquals(category.categoryName, 'Cate2')

		category = getCategoryByCategoryPath('Cate1//', exist=True)
		self.assertEquals(category.categoryName, 'Cate1')

		category = getCategoryByCategoryPath('', exist=True)
		self.assertEquals(category.categoryName, '')
	def test_IsEmpty(self):
		category = Category()
		self.assert_(category.isEmpty())
		page = Page.createPage(category, 'page', 'text')
		self.failIf(category.isEmpty())
		page.remove()
		self.assert_(category.isEmpty())
		child = category.createCategory('child')
		self.failIf(category.isEmpty())
		child.remove()
		self.assert_(category.isEmpty())

	def test_GetPage(self):
		category = Category()
		pageName = 'Test Page'
		rawText = 'life is egg.'
		Page.createPage(category, pageName, rawText)
		page = category.getPage(pageName)
		self.assertEquals(page.pageName, pageName)
		self.assertEquals(page.rawText, rawText)

		categoryName = 'Python'
		child = category.createCategory(categoryName)

		pageName = 'FrontPage'
		rawText = 'life is ddong'
		Page.createPage(child, pageName, rawText)
		page = category.getPage(categoryName)
		self.assertEquals(page.pageName, pageName)
		self.assertEquals(page.rawText, rawText)

		page = category.getPage(categoryName)
		self.failIf(category.getPage('NotExist'))

	def test_GetPageListFromDescendants(self):
		category = Category()
		self.assertEquals(category.getPageListFromDescendants(), [])

		Page.createPage(category, 'Ham', 'text')
		self.assertEquals(len(category.getPageListFromDescendants()), 1)
		
		note = category.createCategory('NoteBook')
		Page.createPage(note, 'note', 'text')
		self.assertEquals(len(category.getPageListFromDescendants()), 2)
		self.assertEquals(len(note.getPageListFromDescendants()), 1)

		morning = note.createCategory('Morning')
		Page.createPage(morning, 'MorningGlory', 'text')
		self.assertEquals(len(category.getPageListFromDescendants()), 3)
		self.assertEquals(len(note.getPageListFromDescendants()), 2)
		self.assertEquals(len(morning.getPageListFromDescendants()), 1)
		
		for i in range(3):
			Page.createPage(morning, 'Han Eun Su %s' % i, 'text')
		for i in range(5):
			Page.createPage(note, 'Han Ji Su %s' % i, 'text')
		myFilter = lambda page:page.pageName.startswith('Han')
		self.assertEquals(len(category.getPageListFromDescendants(myFilter)), 8)
	def test_LoadGroup(self):
		category = Category()
		self.assertEquals(category.group, [])
		category._editGroup(['jangc'])
		category._loadGroup()
		self.assertEquals(category.group, ['jangc'])

	def test_EditGroup(self):
		category = Category()
		self.assertEquals(category.group, [])
		category._editGroup('JangC LeeC KimC'.split())
		self.assertEquals(category.group, ['JangC', 'LeeC', 'KimC'])

		category = Category()
		self.assertEquals(category.group, ['JangC', 'LeeC', 'KimC'])
	def test_GetGroup(self):
		category = Category()
		self.assertEquals(category.getGroup(), [])
		category._editGroup('JangC LeeC KimC'.split())
		self.assertEquals(category.getGroup(), ['JangC', 'LeeC', 'KimC'])
		child = category.createCategory('Child')
		self.assertEquals(child.getGroup(), ['JangC', 'LeeC', 'KimC'])
		child._editGroup('Fishman'.split())
		self.assertEquals(child.getGroup(), ['Fishman'])
		child._editGroup(''.split())
		self.assertEquals(child.getGroup(), ['JangC', 'LeeC', 'KimC'])

	def test_EditAttr(self):
		category = Category()
		category._editGroup('JangC LeeC KimC'.split())
if __name__ == '__main__':
	unittest.main(argv=('','-v'))
