# -*- coding: utf8 -*-
import unittest
import shutil
import os
import time

from Page import *
import Repository
import Category
import Permission
import TagManager

class PageTestCase(unittest.TestCase):
	def setUp(self):
		try: Repository.removeRepository()
		except: pass
		Repository.createRepository()

		self.pageName = 'First Page'
		self.rawText = 'Hello World'
		self.category = Category.Category()
		self.filePath = os.path.join(self.category.dirPath, utils.encode(self.pageName))

	def tearDown(self):
		Repository.removeRepository()

	def test_createPage(self):
		if os.path.exists(self.filePath):
			os.remove(self.filePath)
		page = createPage(self.category, self.pageName, self.rawText)
		self.assert_(isinstance(page, Page))
		self.assert_(os.path.isfile(self.filePath))
	def test_CreatedPagePermission(self):
		page = createPage(self.category, self.pageName, self.rawText, perms='111000')
		self.assertEquals(page.perms.toString(), '111000')
		self.assertEquals(page.perms, '111000')

		os.remove(self.filePath)

		page = createPage(self.category, self.pageName, self.rawText, perms=Permission.Permission('000111'))
		self.assertEquals(page.perms.toString(), Permission.Permission('000111'))
		self.assertEquals(page.perms, Permission.Permission('000111'))

class PageClassTestCase(unittest.TestCase):
	def setUp(self):
		try: Repository.removeRepository()
		except: pass
		Repository.createRepository()

		self.pageName = 'Hello World'
		self.rawText = "I'm your father"
		self.category = Category.Category()
		filePath = os.path.join(self.category.dirPath, utils.encode(self.pageName))
		if os.path.exists(filePath):
			os.remove(filePath)
		self.assert_(createPage(self.category, self.pageName, self.rawText))

	def tearDown(self):
		Repository.removeRepository()

	def test_Init(self):
		page = Page(self.category, self.pageName)
		self.assertEquals(page.pageName, self.pageName)
		self.assertEquals(page.category, self.category)
		self.assertEquals(page.tags, '')
		self.assertEquals(page.encPageName, utils.encode(self.pageName))
		self.assert_(os.path.isfile(page.filePath))
		self.assertEquals(page.rawText, self.rawText)
		
		pageName = 'King Kong'
		tags = 'Movie, Animal'
		rawText = 'so pretty!'
		createPage(self.category, pageName, rawText, tags)
		page = Page(self.category, pageName)
		self.assertEquals(page.tags, tags)

		
	def test_Load(self):
		page = Page(self.category, self.pageName)
		self.assert_(os.path.isfile(page.filePath))
		page.load()
		self.assertEquals(page.rawText, self.rawText)
		#self.assertEquals(page.category.categoryPath, self.category.categoryPath)

	def test_Save(self):
		pageName = 'Page 1'
		rawText = 'raw text'
		editedText = 'edited'
		createPage(self.category, pageName, rawText)
		page = Page(self.category, pageName)
		page.rawText = editedText
		page._save()
		category = Category.Category()
		newPage = category.getPage(pageName)
		self.assertEquals(newPage.rawText, editedText)

	def test_ModifiedTime(self):
		pageName = 'Page 1'
		rawText = 'raw text'
		editedText = 'edited'
		page = createPage(self.category, pageName, rawText)
		mtime1 = str(os.path.getmtime(page.filePath))
		self.assertEquals(page.mtime, mtime1)
		time.sleep(1)
		page._save()
		mtime2 = str(os.path.getmtime(page.filePath))
		self.assertEquals(page.mtime, mtime2)

	def test_Edit(self):
		pageName = 'Page 1'
		rawText = 'raw text'
		editedText = 'edited'
		tags = 'tag1, tag2'
		page = createPage(self.category, pageName, rawText, tags)
		self.assertEquals(page.rawText, rawText)
		tagSet = TagManager.TagManager(tags).toSet()
		self.assertEquals(len(tagSet), 2)
		for tagName in tagSet:
			self.assert_(pageName in Tag(tagName).getPagePathList())
		
		editedText = 'editedText'
		editedTags = 'tag2, tag3'
		page.edit(editedText, editedTags)
		self.assertEquals(page.rawText, editedText)
		tagSet = TagManager.TagManager(editedTags).toSet()
		self.assertEquals(len(tagSet), 2)
		for tagName in tagSet:
			self.assert_(pageName in Tag(tagName).getPagePathList())
		self.assert_(pageName not in Tag('tag1').getPagePathList())
		
	def test_Remove(self):
		page = Page(self.category, self.pageName)
		self.assert_(os.path.exists(page.filePath))
		self.assert_(page.remove())
		self.failIf(os.path.exists(page.filePath))
		self.assertRaises(NotExistPage, page.remove)
		
		child = self.category.createCategory('Child')
		page1 = createPage(child, 'Page1', 'text', 'CinemaJD')
		page2 = createPage(child, 'Page2', 'text', 'CinemaJD')
		self.assert_(os.path.exists(Tag('CinemaJD').tagPath))
		self.assert_(page1.remove())
		self.failIf(os.path.exists(page1.filePath))
		self.assert_(os.path.exists(Tag('CinemaJD').tagPath))
		self.assert_(os.path.exists(child.dirPath))
		self.assert_(page2.remove())
		self.failIf(os.path.exists(page2.filePath))
		self.failIf(os.path.exists(Tag('CinemaJD').tagPath))
		self.failIf(os.path.exists(child.dirPath))
		


	def test_Backup(self): # FIXME timeStamp관련 코드 추가할 것.
		pageName = 'Page 1'
		rawText = 'raw text'
		page = createPage(self.category, pageName, rawText)
		page.backup()
		backupRepository = Repository.getPageBackupRepository()
		cur, dirs, files = os.walk(backupRepository).next()
		mtime = str(int(os.path.getmtime(page.filePath)))
		self.assertEquals(files, ['Page_201.%s' % mtime])

		childCategory = self.category.createCategory('QueSeraSera')
		pageName = 'Page 2'
		rawText = 'raw text'
		page = createPage(childCategory, pageName, rawText)
		page.backup()
		backupRepository = os.path.join(backupRepository, utils.encode(childCategory.categoryName))
		cur, dirs, files = os.walk(backupRepository).next()
		mtime = str(int(os.path.getmtime(page.filePath)))
		self.assertEquals(files, ['Page_202.%s' % mtime])
	def test_GetGroup(self):
		pageName = 'Page 1'
		rawText = 'raw text'
		group = ['jangc', 'python']
		self.category._editGroup(group)
		page = createPage(self.category, pageName, rawText)
		self.assertEquals(page.getGroup(), group)

	def test_GetPage_GetTemplate(self):
		self.assert_(self.category.createCategory(config.templateCategoryName))
		self.assertEquals(getPage(config.templateCategoryName, ''), None)

	def test_GetPageTravelAlongRails(self):
		root = Category.Category()

		pageName = 'GoodEgg'
		rawText = 'Life is egg'
		createPage(root, pageName, rawText)
		page = travelAlongRails(root.categoryPath, pageName)
		self.assertEquals(page.pageName, pageName)

		first = root.createCategory('First')
		pageName = 'GoodSpam'
		createPage(first, pageName, rawText)
		page = travelAlongRails(first.categoryPath, pageName)
		self.assertEquals(page.pageName, pageName)

		page = travelAlongRails(root.categoryPath, pageName)
		self.assertEquals(page.pageName, pageName)

		
		page = travelAlongRails(root.categoryPath, 'First')
		self.failIf(page)
		
		pageName = 'FrontPage'
		createPage(first, pageName, rawText)
		page = travelAlongRails(root.categoryPath, 'First')
		self.assertEquals(page.category.categoryPath, first.categoryPath)
		self.assertEquals(page.pageName, pageName)

		pageName = 'GoodPotato'
		second = first.createCategory('Second')
		createPage(second, pageName, rawText)
		page = travelAlongRails(root.categoryPath, pageName)
		self.assertEquals(page.category.categoryPath, second.categoryPath)
		self.assertEquals(page.pageName, pageName)
		page = travelAlongRails(first.categoryPath, pageName)
		self.assertEquals(page.category.categoryPath, second.categoryPath)
		self.assertEquals(page.pageName, pageName)

		pageName = 'FrontPage'
		createPage(second, pageName, rawText)
		page = travelAlongRails(root.categoryPath, 'Second')
		self.assertEquals(page.category.categoryPath, second.categoryPath)
		self.assertEquals(page.pageName, pageName)

		pageName = 'TitleIndex'
		page = travelAlongRails(second.categoryPath, pageName)
		self.failIf(page) 
		globalCategory = root.createCategory(config.globalCategoryName) 
		createPage(globalCategory, pageName, rawText)
		page = travelAlongRails(second.categoryPath, pageName)
		self.assertEquals(page.pageName, pageName)
		self.assertEquals(page.category.categoryName, globalCategory.categoryName)

		createPage(globalCategory, 'FrontPage', rawText)
		emptyCategory = root.createCategory('Empty')
		page = travelAlongRails('', 'Empty')
		self.assertEquals(page.pageName, 'FrontPage')
		self.assertEquals(page.category.categoryPath, 'Global')
	def test_CreateAttachmentFolder(self):
		root = Category.Category()
		pageName = 'GoodEgg'
		rawText = 'Life is egg'
		page = createPage(root, pageName, rawText)

		self.assert_(page.createAttachmentFolder())
		self.failIf(page.createAttachmentFolder())
	def test_AttachmentFolderIsExist(self):
		root = Category.Category()
		pageName = 'GoodEgg'
		rawText = 'Life is egg'
		page = createPage(root, pageName, rawText)

		self.failIf(page.isExistAttachmentFolder())
		self.assert_(page.createAttachmentFolder())
		self.assert_(page.isExistAttachmentFolder())

	def test_Attach(self):
		root = Category.Category()
		pageName = 'GoodEgg'
		rawText = 'Life is egg'
		page = createPage(root, pageName, rawText)
		fileName = 'attachment'
		value = 'apple'
		newFileName = page.attach(fileName, value)
		self.assertEquals(newFileName, fileName)
		attachmentPath = os.path.join(page.attachmentFolderPath, fileName)
		self.assert_(os.path.exists(attachmentPath))
		self.assertEquals(file(attachmentPath, 'rb').read(), value)

		newFileName = page.attach(fileName, value)
		self.assertEquals(newFileName, fileName+'_')
		newFileName = page.attach(fileName, value)
		self.assertEquals(newFileName, fileName+'__')

	def test_Attach(self):
		root = Category.Category()
		pageName = 'GoodEgg'
		rawText = 'Life is egg'
		page = createPage(root, pageName, rawText)
		fileName = 'star wars'
		value = 'apple'
		newFileName = page.attach(fileName, value)
		self.assertEquals(newFileName, 'star_wars')

	def test_GetAttachedFiles(self):
		root = Category.Category()
		pageName = 'GoodEgg'
		rawText = 'Life is egg'
		page = createPage(root, pageName, rawText)
		self.assertEquals(page.getAttachedFiles(), [])
		fileName = 'attachment'
		value = 'apple'
		newFileName = page.attach(fileName, value)
		self.assertEquals(page.getAttachedFiles(), ['attachment'])

	def test_GetAttachment(self):
		root = Category.Category()
		pageName = 'GoodEgg'
		rawText = 'Life is egg'
		page = createPage(root, pageName, rawText)

		self.assertEquals(page.getAttachment('NotExist'), None)

		fileName = 'attachment'
		value = 'apple\norange\n'
		page.attach(fileName, value)
		self.assertEquals(page.getAttachment(fileName), value)

	def test_BackupAttachment(self):
		root = Category.Category()
		pageName = 'GoodEgg'
		rawText = 'Life is egg'
		page = createPage(root, pageName, rawText)
		fileName = 'attachment'
		value = 'apple\norange\n'
		page.attach(fileName, value)
		self.failIf(os.path.exists(os.path.join(Repository.getAttachmentBackupRepository(), utils.encode(page.pagePath), fileName)))
		timeStamp = str(int(time.time()))
		newFileName = page.backupAttachment(fileName, timeStamp)
		self.assert_(os.path.exists(os.path.join(Repository.getAttachmentBackupRepository(), utils.encode(page.pagePath), newFileName)))

		self.assertRaises(NotExistAttachment, page.backupAttachment, 'NotExist', timeStamp)
	def test_RemoveAttachment(self):
		root = Category.Category()
		pageName = 'GoodEgg'
		rawText = 'Life is egg'
		page = createPage(root, pageName, rawText)
		fileName = 'attachment'
		value = 'apple\norange\n'
		page.attach(fileName, value)
		self.assert_(page.isExistAttachment(fileName))
		page.removeAttachment(fileName)
		self.failIf(page.isExistAttachment(fileName))

		self.assertRaises(NotExistAttachment, page.removeAttachment, fileName)

if __name__ == '__main__':
	unittest.main(argv=('','-v'))
